/*
 *  Dr.Mario port for Uzebox ver 1.1
 *  Copyright (C) 2009  Codecrank, Mapes
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Uzebox is a reserved trade mark
*/

#include <stdlib.h>
#include "data/data.c"

typedef int bool; enum {false,true};

enum {BTN_START=8,BTN_A=1,BTN_B,BTN_LEFT=2,BTN_RIGHT=4,BTN_UP,BTN_DOWN=8};
extern int readkey ();
int ReadJoypad (int j) {
   switch (readkey()) {
      case 0x1A: return BTN_A;
      case 0x04: return BTN_LEFT;
      case 0x16: return BTN_DOWN;
      case 0x07: return BTN_RIGHT;
   }
   return 0;
}

void ClearVsyncFlag() {}
extern void vblank();
int GetVsyncFlag() {vblank(); return 1;}

int * sprites;
void SetTileTable (int * p) {sprites = p;}

extern void putpixel (int x, int y, int color);
extern void tile (int x, int y, int *p);

void SetTile (int x, int y, int n) {
   tile(6*x,y<<3,sprites+(n<<4));
}

void Fill(int x, int y, int w, int h, int c) {
   for (int i=x;i<x+w;i++)
   for (int j=y;j<y+h;j++) SetTile(i,j,c);
}

int textx = 0, texty = 0;

void chr6x8 (int n) {
   int * p = fonts + 3*n;
   int c = ((n+0x20)<':')?14:15;
   for (int j=0;j<8;j++)
   for (int i=0;i<6;i++) {
      putpixel (textx+i,texty+j,((p[i>>1] >> (((i&1)<<3)+j))&1)*c);
   }
}

void printc ( char c ) {
   chr6x8(c-0x20);
   textx += 6;
}

void printd (char d) {
   int n;
   printc((n=d&15)+((n<10)?0x30:0x37));
}

void PrintByte (int b, int x, int y) {
   textx = 6*x;
   texty = 8*y;
   printd(b>>4);
   printd(b);
}

void Print (char * s, int x, int y) {
   textx = 6*x;
   texty = 8*y;
   char *p = s;
   while (*p) printc(*p++);
}

void DrawMap (char * m) {
   int * p = m, i;
   for (int y=0;y<24;y++)
   for (int x=0;x<20;x++) {
      SetTile (2*x,y,((i=*p++)>>8)&0xFF);
      SetTile (2*x+1,y,i&0xFF);
   }
}




enum {MOVE_LEFT=1,MOVE_RIGHT=2,MOVE_DOWN=3,MOVE_SPIN=4,MOVE_START=3};

enum {PLAYER1, PLAYER2};

enum {
   BOTTLE_FIRST_COLUMN   = 4,
   BOTTLE2_FIRST_COLUMN  = 28,
   BOTTLE_LAST_COLUMN    = 11,
   BOTTLE_FIRST_ROW      = 5,
   BOTTLE_LAST_ROW       = 21,
   BOTTLE_WIDTH          = 8,
   BOTTLE_DEPTH          = 17
};

enum {
   VIRUS_LEFT_ROW        = 22,
   VIRUS_LEFT_COLUMN1    = 17,
   VIRUS_LEFT_COLUMN2    = 21
};

enum {
   LEFT_RED              =  1,
   LEFT_BLUE             =  3,
   LEFT_YELLOW           =  5,
   RIGHT_RED             =  2,
   RIGHT_BLUE            =  4,
   RIGHT_YELLOW          =  6,
   BOTTOM_RED            =  7,
   BOTTOM_BLUE           =  9,
   BOTTOM_YELLOW         = 11,
   TOP_RED               =  8,
   TOP_BLUE              = 10,
   TOP_YELLOW            = 12,
   SINGLE_RED            = 13,
   SINGLE_BLUE           = 14,
   SINGLE_YELLOW         = 15,

   VIRUS_RED1            = 16,
   VIRUS_RED2            = 17,
   VIRUS_BLUE1           = 18,
   VIRUS_BLUE2           = 19,
   VIRUS_YELLOW1         = 20,
   VIRUS_YELLOW2         = 21,

   DISSAPEAR_ANIM_1      = 22,
   DISSAPEAR_ANIM_2      = 23,
   DISSAPEAR_ANIM_3      = 24,
   DISSAPEAR_ANIM_4      = 25,
   DISSAPEAR_ANIM_5      =  0
};

enum {HORIZ,VERT};

const char * strPress = "PRESS";
const char * strStart = "START";
const char * strBlank = "     ";

const char * strWinner = "WINNER!";
const char * strLoser = "LOSER!";

const char * strWin = "WIN";

struct pill {
        bool orientation;
        unsigned short length;
        unsigned short x;
        unsigned short y;
        unsigned short bottle_x;
        unsigned short bottle_y;
        unsigned short top;
        unsigned short bottom;
        unsigned short left;
        unsigned short right;
};

struct bottle {
        unsigned short space[8][17];
        unsigned short virus_count;
        bool check_for_sequences;
        unsigned short sequences_count;
        bool contains_floaters;
        unsigned short dissapear_anim;
        unsigned short qued_trash;
        bool new_pill;
};


struct pill current_pill[2];
struct pill next_pill[2];
unsigned short gravity=60;
unsigned int round_time;
unsigned short auto_repeat_counter=0;
struct bottle bottle[2];
unsigned short virus_anim_counter=0;
unsigned short drop_floater_timer=0;
unsigned short x,y,j,k,saved_state, z=0; // generic
unsigned char difficulty_level[] = {1,1};
bool round_lost[2];
unsigned short rounds_won[2];
void SetStageSettings();
unsigned short move_type=0;
//unsigned short seed;
void DissapearAnim(unsigned short player);
//void RoundOver();
unsigned int long_timer;

void SetStageSettings()
{

        unsigned int c;

        SetTileTable(main_board);
        DrawMap(drmario_map_stage_select);

        Print("2 PLAYER GAME",14, 7);
        Print("LEVEL",7,9);

//        SetTile(difficulty_level[PLAYER1]+8,6,53);
//        SetTile(difficulty_level[PLAYER2]+8,8,54);

//        PrintByte(difficulty_level[PLAYER1],31,6);
//        PrintByte(difficulty_level[PLAYER2],31,8);

        do {

                PrintByte(difficulty_level[PLAYER1],31,11);
                PrintByte(difficulty_level[PLAYER2],31,13);

                SetTile(difficulty_level[PLAYER1]+8,11,53);
                SetTile(difficulty_level[PLAYER2]+8,13,54);

                if (c = ReadJoypad(PLAYER1)) while (ReadJoypad(PLAYER1));


                if ( ( c&BTN_RIGHT ) && ( difficulty_level[PLAYER1] < 20 ) )
                {
                        SetTile((difficulty_level[PLAYER1]++)+8,11,0);
                }
                else if ( ( c&BTN_LEFT) && ( difficulty_level[PLAYER1] > 1 ) )
                {
                        SetTile((difficulty_level[PLAYER1]--)+8,11,0);
                }

/*
                if (c = ReadJoypad(PLAYER2)) while (ReadJoypad(PLAYER2));

                if ( (  c&BTN_RIGHT ) && ( difficulty_level[PLAYER2] < 20 ) )
                {
                        SetTile(difficulty_level[PLAYER2]+8,8,0);
                        SetTile(++difficulty_level[PLAYER2]+8,8,54);
                        PrintByte(difficulty_level[PLAYER2],31,8);
                }
                else if ( ( c&BTN_LEFT ) && ( difficulty_level[PLAYER2] > 1 ) )
                {
                        SetTile(difficulty_level[PLAYER2]+8,8,0);
                        SetTile(--difficulty_level[PLAYER2]+8,8,54);
                        PrintByte(difficulty_level[PLAYER2],31,8);
                }
*/

        } while (!(c&BTN_START));


}

        static bool holdKey[2];
unsigned short grab_input(unsigned short player)
{
        unsigned int c = ReadJoypad(player);

        //check if spin key was released
        if(!(c&BTN_A)) holdKey[player]=false;

        if ( auto_repeat_counter++ < 4 ) return 0;
        auto_repeat_counter=0;

        if (c&BTN_A && holdKey[player]==false)
        {
                holdKey[player]=true;
                return MOVE_SPIN;

        }
        else if ( c&BTN_RIGHT ) return MOVE_RIGHT;
        else if ( c&BTN_LEFT )  return MOVE_LEFT;
//        else if ( c&BTN_START ) return MOVE_START;
        else if ( c&BTN_DOWN )  return MOVE_DOWN;

        return 0;

}


void DissapearAnim(unsigned short player)
{
        bottle[player].dissapear_anim=0;

        unsigned short first_column = player ? BOTTLE2_FIRST_COLUMN : BOTTLE_FIRST_COLUMN ;

        for ( x=0 ; x< BOTTLE_WIDTH ; x++ )
        {
                for ( y=0 ; y< BOTTLE_DEPTH ; y++ )
                {

                        if ( (  bottle[player].space[x][y] >= DISSAPEAR_ANIM_1 ) && ( bottle[player].space[x][y] <= DISSAPEAR_ANIM_3 ) )
                        {
                                SetTile(first_column+x, BOTTLE_FIRST_ROW+y, ++bottle[player].space[x][y]);
                                bottle[player].dissapear_anim=1;
                        }
                        else if ( bottle[player].space[x][y]  == DISSAPEAR_ANIM_4 )
                        {
                                SetTile(first_column+x, BOTTLE_FIRST_ROW+y, bottle[player].space[x][y]=0);
                                bottle[player].dissapear_anim=1;
                        }
                }
        }
}


void NextPill(unsigned short player)
{
//   next_pill[player].x = 24*player + 7;
//   next_pill[player].y = BOTTLE_FIRST_ROW -2 ;

   SetTile(next_pill[player].x  = 24*player + 7,next_pill[player].y = BOTTLE_FIRST_ROW -2 , next_pill[player].left  = rand()%3*2 + 1);
   SetTile(next_pill[player].x+1,next_pill[player].y, next_pill[player].right = rand()%3*2 + 2);

}

void NewPill(unsigned short player)
{


// check if bottle if full

        if ( ( bottle[player].space[3][0] ) || ( bottle[player].space[4][0] ) )
        {
                round_lost[player]=1;
                return;
        }

        current_pill[player].x = 24*player + 7;

        current_pill[player].bottle_x=3;
        current_pill[player].y = BOTTLE_FIRST_ROW;
        current_pill[player].bottle_y=0;

        current_pill[player].orientation = 0;
        current_pill[player].length = 2;

        current_pill[player].left = next_pill[player].left;
        current_pill[player].right = next_pill[player].right;

        SetTile(current_pill[player].x,current_pill[player].y, current_pill[player].left);
        SetTile(current_pill[player].x+1,current_pill[player].y, current_pill[player].right);

        NextPill(player);

}

void LockPill(unsigned short player)
{
        if ( current_pill[player].orientation )
        {
                // use horizontal tiles for saving color
                bottle[player].space[current_pill[player].bottle_x][current_pill[player].bottle_y-1]=current_pill[player].top;
                bottle[player].space[current_pill[player].bottle_x][current_pill[player].bottle_y]=current_pill[player].bottom;
        }
        else
        {
                bottle[player].space[current_pill[player].bottle_x][current_pill[player].bottle_y]=current_pill[player].left;
                bottle[player].space[current_pill[player].bottle_x+1][current_pill[player].bottle_y]=current_pill[player].right;
        }

        bottle[player].check_for_sequences=1;

}

bool MovePill(unsigned short player, unsigned short move_type);

bool MoveDown (unsigned short player) {
        bool moved=0;
switch (moved) { default:
                        if ( current_pill[player].y ==  BOTTLE_LAST_ROW )
                        {
                                LockPill(player);
                                bottle[player].new_pill = true;

                                break;
                        }

                        if ( current_pill[player].orientation )
                        {
                                //  something below me ?
                                if ( bottle[player].space[current_pill[player].bottle_x][current_pill[player].bottle_y+1] )
                                {
                                        LockPill(player);
                                        bottle[player].new_pill = true;
                                }
                                else
                                {
                                        SetTile(current_pill[player].x, (++current_pill[player].y - current_pill[player].length), 0);
                                        current_pill[player].bottle_y++;
                                        SetTile(current_pill[player].x,current_pill[player].y-1, current_pill[player].top);
                                        SetTile(current_pill[player].x,current_pill[player].y,  current_pill[player].bottom);
                                        moved=1;
                                }

                        }
                        else
                        {
                                if (  bottle[player].space[current_pill[player].bottle_x][current_pill[player].bottle_y+1]
                                        ||  bottle[player].space[current_pill[player].bottle_x+1][current_pill[player].bottle_y+1]  )
                                {

                                        LockPill(player);
                                        bottle[player].new_pill = true;


                                }
                                else
                                {

                                        Fill(current_pill[player].x,current_pill[player].y++, current_pill[player].length, 1, 0);
                                        current_pill[player].bottle_y++;
                                        SetTile(current_pill[player].x,current_pill[player].y,  current_pill[player].left);
                                        SetTile(current_pill[player].x+1,current_pill[player].y,  current_pill[player].right);
                                        moved=1;
                                }
                        }



                        break;
}
return moved;
}

bool MoveLeft (unsigned short player) {
        bool moved=0;
switch (moved) { default:

                        // first column , can't move left
                        if ( (current_pill[player].bottle_x == 0) )
                                break ;


                        if ( current_pill[player].orientation )
                        {
                                //  nothing left of me ?
                                if ( ! ( bottle[player].space[current_pill[player].bottle_x-1][current_pill[player].bottle_y]
                                         || bottle[player].space[current_pill[player].bottle_x-1][current_pill[player].bottle_y-1] ) )
                                {


                                        Fill(current_pill[player].x--, current_pill[player].y-1, 1,current_pill[player].length , 0);
                                        current_pill[player].bottle_x--;
                                        SetTile(current_pill[player].x,current_pill[player].y-1, current_pill[player].top);
                                        SetTile(current_pill[player].x,current_pill[player].y, current_pill[player].bottom);
                                        moved=1;
                                }
                        }
                        else
                        {
                                //  nothing left of me ?
                                if ( ! bottle[player].space[current_pill[player].bottle_x-1][current_pill[player].bottle_y] )
                                {

                                        Fill(current_pill[player].x--,current_pill[player].y, current_pill[player].length, 1, 0);
                                        current_pill[player].bottle_x--;
                                        SetTile(current_pill[player].x,current_pill[player].y, current_pill[player].left);
                                        SetTile(current_pill[player].x+1,current_pill[player].y, current_pill[player].right);
                                        moved=1;
                                }
                        }

                        break;

}
return moved;
}

bool MoveRight (unsigned short player) {
        bool moved=0;
switch (moved) { default:

                        if ( current_pill[player].orientation )   // VERTICAL
                        {

                                if (current_pill[player].bottle_x == (BOTTLE_WIDTH - 1) )
                                        break;

                                //  nothing right of me ?
                                if ( ! ( bottle[player].space[current_pill[player].bottle_x+1][current_pill[player].bottle_y]
                                                || bottle[player].space[current_pill[player].bottle_x+1][current_pill[player].bottle_y-1])  )
                                {


                                        Fill(current_pill[player].x++, current_pill[player].y-1 , 1, current_pill[player].length, 0);
                                        current_pill[player].bottle_x++;
                                        SetTile(current_pill[player].x,current_pill[player].y-1, current_pill[player].top);
                                        SetTile(current_pill[player].x,current_pill[player].y, current_pill[player].bottom);
                                        moved=1;
                                }

                        }
                        else
                        {
                                if  ( current_pill[player].bottle_x  == (BOTTLE_WIDTH - 2) )
                                        break;

                                //  nothing right of me ?
                                if ( ! bottle[player].space[current_pill[player].bottle_x+2][current_pill[player].bottle_y] )
                                {

                                        SetTile(current_pill[player].x++, current_pill[player].y, 0);
                                        current_pill[player].bottle_x++;
                                        SetTile(current_pill[player].x,current_pill[player].y, current_pill[player].left);
                                        SetTile(current_pill[player].x+1,current_pill[player].y, current_pill[player].right);
                                        moved=1;
                                }

                        }

                        break;

}
return moved;
}

bool MoveSpin (unsigned short player) {
        bool moved=0;
switch (moved) { default:

                        if ( current_pill[player].orientation ) // VERTICAL
                        {

                                // check for possible wall kicks to the left

                                // against the wall or pils ?
                                if  (  ( current_pill[player].bottle_x == BOTTLE_WIDTH-1  )
                                         ||( bottle[player].space[current_pill[player].bottle_x+1][current_pill[player].bottle_y]) )
                                {
                                                if ( ! MovePill(player, MOVE_LEFT) )
                                                {
                                                        // special squeeze kick possible ?
                                                        if (  bottle[player].space[current_pill[player].bottle_x-1][current_pill[player].bottle_y] == 0 )
                                                        {
                                                                // can't go through bottle wall...
                                                                if ( current_pill[player].bottle_x == 0 )
                                                                        break;


                                                                SetTile (current_pill[player].x--,current_pill[player].y-1,0);

                                                                current_pill[player].bottle_x--;
                                                                SetTile(current_pill[player].x,current_pill[player].y, current_pill[player].left =  current_pill[player].top - 7);
                                                        SetTile(current_pill[player].x+1,current_pill[player].y, current_pill[player].right = current_pill[player].bottom - 5);

                                                                current_pill[player].orientation = HORIZ;
                                                                moved=1;

                                                        }

                                                        break;

                                                }

                                }




                                Fill(current_pill[player].x, ( current_pill[player].y - 1 ), 1, 2, 0);

                                SetTile(current_pill[player].x,current_pill[player].y,  current_pill[player].left =  current_pill[player].top - 7);
                                SetTile(current_pill[player].x+1,current_pill[player].y, current_pill[player].right = current_pill[player].bottom - 5);

                                current_pill[player].orientation = HORIZ;
                                moved=1;
                        }
                        else
                        {

                                // check for possible ceiling kicks
                                if (  current_pill[player].bottle_y == 0 )
                                {
                                                if ( ! MovePill(player, MOVE_DOWN) )
                                                        break ;
                                }
                                if  ( bottle[player].space[current_pill[player].bottle_x][current_pill[player].bottle_y-1]  )
                                {
                                                if ( ! MovePill(player, MOVE_DOWN) )
                                                        break;
                                }

                                Fill(current_pill[player].x,current_pill[player].y, 2, 1, 0);

                                SetTile(current_pill[player].x,current_pill[player].y-1, current_pill[player].top = current_pill[player].right + 6);
                                SetTile(current_pill[player].x,current_pill[player].y, current_pill[player].bottom = current_pill[player].left + 6);

                                moved=1;

                                current_pill[player].orientation = VERT;
                        }

//                        break;


}
return moved;
}

bool MovePill(unsigned short player, unsigned short move_type)
{

        bool moved=0;

        switch (move_type ) {


                case MOVE_DOWN:  moved = MoveDown(player); break;

                case MOVE_LEFT:  moved = MoveLeft(player); break;

                case MOVE_RIGHT: moved = MoveRight(player); break;

                case MOVE_SPIN:  moved = MoveSpin(player);

        }


        return moved;

}

void SplitPill(unsigned short player, unsigned short x , unsigned short y)
{

        unsigned short first_column = player ? BOTTLE2_FIRST_COLUMN : BOTTLE_FIRST_COLUMN;

        bottle[player].check_for_sequences=0;

        switch ( bottle[player].space[x][y] )
        {
                // half pills
//                case SINGLE_RED: case SINGLE_BLUE: case SINGLE_YELLOW:
//                        break;

                // left half
                case LEFT_RED:
                case LEFT_BLUE:
                case LEFT_YELLOW:
                        switch (bottle[player].space[x+1][y])
                        {
                                // right half
                                case RIGHT_RED:
                                        bottle[player].space[x+1][y]=SINGLE_RED;
                                        SetTile(first_column+x+1,BOTTLE_FIRST_ROW+y,SINGLE_RED);
                                        break;
                                case RIGHT_BLUE:
                                        bottle[player].space[x+1][y]=SINGLE_BLUE;
                                        SetTile(first_column+x+1,BOTTLE_FIRST_ROW+y,SINGLE_BLUE);
                                        break;
                                case RIGHT_YELLOW:
                                        bottle[player].space[x+1][y]=SINGLE_YELLOW;
                                        SetTile(first_column+x+1,BOTTLE_FIRST_ROW+y,SINGLE_YELLOW);
//                                        break;
                        }
                        break;

                case RIGHT_RED:
                case RIGHT_BLUE:
                case RIGHT_YELLOW:
                        switch (bottle[player].space[x-1][y])
                        {

                                case LEFT_RED:
                                        bottle[player].space[x-1][y]=SINGLE_RED;
                                        SetTile(first_column+x-1,BOTTLE_FIRST_ROW+y,SINGLE_RED);
                                        break;
                                case LEFT_BLUE:
                                        bottle[player].space[x-1][y]=SINGLE_BLUE;
                                        SetTile(first_column+x-1,BOTTLE_FIRST_ROW+y,SINGLE_BLUE);
                                        break;
                                case LEFT_YELLOW:
                                        bottle[player].space[x-1][y]=SINGLE_YELLOW;
                                        SetTile(first_column+x-1,BOTTLE_FIRST_ROW+y,SINGLE_YELLOW);
//                                        break;
                        }
                        break;

                case BOTTOM_RED:
                case BOTTOM_BLUE:
                case BOTTOM_YELLOW:
                        switch (bottle[player].space[x][y-1])
                        {
                                // right half
                                case TOP_RED:
                                        bottle[player].space[x][y-1]=SINGLE_RED;
                                        SetTile(first_column+x,BOTTLE_FIRST_ROW+y-1,SINGLE_RED);
                                        break;
                                case TOP_BLUE:
                                        bottle[player].space[x][y-1]=SINGLE_BLUE;
                                        SetTile(first_column+x,BOTTLE_FIRST_ROW+y-1,SINGLE_BLUE);
                                        break;
                                case TOP_YELLOW:
                                        bottle[player].space[x][y-1]=SINGLE_YELLOW;
                                        SetTile(first_column+x,BOTTLE_FIRST_ROW+y-1,SINGLE_YELLOW);
//                                        break;
                        }
                        break;

                case TOP_RED:
                case TOP_BLUE:
                case TOP_YELLOW:
                        switch (bottle[player].space[x][y+1])
                        {
                                // right half
                                case BOTTOM_RED:
                                        bottle[player].space[x][y+1]=SINGLE_RED;
                                        SetTile(first_column+x,BOTTLE_FIRST_ROW+y+1,SINGLE_RED);
                                        break;
                                case BOTTOM_BLUE:
                                        bottle[player].space[x][y+1]=SINGLE_BLUE;
                                        SetTile(first_column+x,BOTTLE_FIRST_ROW+y+1,SINGLE_BLUE);
                                        break;
                                case BOTTOM_YELLOW:
                                        bottle[player].space[x][y+1]=SINGLE_YELLOW;
                                        SetTile(first_column+x,BOTTLE_FIRST_ROW+y+1,SINGLE_YELLOW);
//                                        break;
                        }
//                        break;
        }

}

unsigned short CheckColorMatch( unsigned short tile1 , unsigned short tile2 )
{
        switch (tile1)
        {

                case 1: case 2:
                case 7: case 8:
                case 13:
                case 16: case 17:
                        // red
                        switch (tile2)
                        {
                                case 1: case 2:
                                case 7: case 8:
                                case 13:
                                case 16: case 17:
                                        return 1;     // RED match
                        }
                        break;

                case 3: case 4:
                case 9: case 10:
                case 14:
                case 18: case 19:
                        // blue
                        switch (tile2)
                        {
                                case 3: case 4:
                                case 9: case 10:
                                case 14:
                                case 18: case 19:
                                        return 1;     // BLUE match
                        }
                        break;

                // must be yellow
                default :
                        switch (tile2)
                        {
                                case 5: case 6:
                                case 11: case 12:
                                case 15:
                                case 20: case 21:
                                        return 1;     // YELLLOW match
                        }
//                        break;
    }

        return 0;
}


void CheckMatching4(unsigned short player)
{

        short unsigned int matching_spaces = 1;
        unsigned short first_column = player ? BOTTLE2_FIRST_COLUMN : BOTTLE_FIRST_COLUMN;
        unsigned short virus_left_column = player ? VIRUS_LEFT_COLUMN2 : VIRUS_LEFT_COLUMN1;

        bottle[player].check_for_sequences=0;

        for ( x=0 ; x< BOTTLE_WIDTH ; x++ )
        {
                for ( y=0 ; y< BOTTLE_DEPTH ; y++ )
                {
                        // for evey non blank space
                        if ( bottle[player].space[x][y] )
                        {

                                // check remaining of column for matching colors
                                matching_spaces=1;
                                for ( j=0; j < BOTTLE_DEPTH ; j++ )
                                {
                                                if ( y+j+1 > BOTTLE_DEPTH )
                                                        break; // don't check passed last row

                                        if ( CheckColorMatch ( bottle[player].space[x][y+j], bottle[player].space[x][y+j+1]) )
                                                {
                                                        matching_spaces++;
                                                }
                                                else
                                                {
                                                        // are we at the end of a matching sequence  ?
                                                        if ( matching_spaces >= 4 )
                                                        {

                                                                // don't need j anymore
                                                                for ( j=0 ; j< matching_spaces ; j++)
                                                                {

                                                                        SplitPill(player,x,y+j);
                                                                        bottle[player].contains_floaters =1;
                                                                        // update virus count is needed
                                                                        if ( ( bottle[player].space[x][y+j] >= VIRUS_RED1 ) && ( bottle[player].space[x][y+j] <= VIRUS_YELLOW2 ) )
                                                                        {
                                                                                PrintByte( --bottle[player].virus_count, virus_left_column, VIRUS_LEFT_ROW);
                                                                        }


                                                                        bottle[player].space[x][y+j]=DISSAPEAR_ANIM_1;
                                                                }

                                                                // start pill dissapear animation sequence
                                                                Fill(first_column+x,BOTTLE_FIRST_ROW+y,1,matching_spaces,DISSAPEAR_ANIM_1);
                                                                bottle[player].dissapear_anim=1;

                                                                bottle[player].check_for_sequences=1;
                                                                bottle[player].sequences_count++;

                                                        }
                                                        // stop searching column
                                                        break;
                                                }
                                }

                                // check remaining of row for matching colors


                                matching_spaces=1;
                                for ( j=0; j < BOTTLE_WIDTH ; j++ )
                                {
                                                if ( x+j+1 > BOTTLE_WIDTH )
                                                        break; // don't check passed last row

                                        if ( CheckColorMatch ( bottle[player].space[x+j][y], bottle[player].space[x+j+1][y]) )
                                                {
                                                        matching_spaces++;
                                                }
                                                else
                                                {
                                                        // are we at the end of a matching sequence  ?
                                                        if ( matching_spaces >= 4 )
                                                        {

                                                                // don't need j anymore, let;s uze it for something
                                                                for ( j=0 ; j< matching_spaces ; j++)
                                                                {
                                                                        // remove pill(s)/parts from bottle
                                                                        SplitPill(player, x+j,y);
                                                                        bottle[player].contains_floaters =1;
                                                                        // update virus count is needed
                                                                        if ( ( bottle[player].space[x+j][y] >= VIRUS_RED1 ) && ( bottle[player].space[x+j][y] <= VIRUS_YELLOW2 ) )
                                                                        {
                                                                                PrintByte( --bottle[player].virus_count, virus_left_column, VIRUS_LEFT_ROW);
                                                                        }



                                                                        bottle[player].space[x+j][y]=DISSAPEAR_ANIM_1;

                                                                }


                                                                // black out matching color sequence
                                                                Fill(first_column+x,BOTTLE_FIRST_ROW+y,matching_spaces,1,DISSAPEAR_ANIM_1);
                                                                bottle[player].dissapear_anim=1;

                                                                bottle[player].check_for_sequences=1;
                                                                bottle[player].sequences_count++;
                                                        }

                                                        // stop searching column

                                                        break;
                                                }
                                }
                        }
                }
        }
}

void QueTrash(unsigned short sender, unsigned short trash_count)
{
//        bottle[1-sender].qued_trash += trash_count;
        bottle[sender].qued_trash += trash_count;
}

void FlingTrash(unsigned short sender)
{
        unsigned short receiver = 1 - sender;
        unsigned short first_column = sender ? BOTTLE_FIRST_COLUMN : BOTTLE2_FIRST_COLUMN;

        z=0;

        while ( z < bottle[sender].qued_trash   )
        {

                x= rand() % BOTTLE_WIDTH ;
                y= 1;

                if ( !  bottle[receiver].space[x][y] )
                {
                        bottle[receiver].space[x][y]= SINGLE_RED + (rand() % 3);

                        z++;
                        SetTile(first_column+x, BOTTLE_FIRST_ROW+y, bottle[receiver].space[x][y]);

                }
        }
        bottle[sender].qued_trash = 0;

        bottle[receiver].contains_floaters=1;

}


void DropFilledSpace(unsigned short player, unsigned short x , unsigned short y)
{

        unsigned short first_column = player ? BOTTLE2_FIRST_COLUMN : BOTTLE_FIRST_COLUMN;

        // move current space 1 dowm
        SetTile(first_column+x,BOTTLE_FIRST_ROW+y+1,bottle[player].space[x][y+1]=bottle[player].space[x][y]);

        // blank out current space
        SetTile(first_column+x,BOTTLE_FIRST_ROW+y,bottle[player].space[x][y]=0);

}



void DropFloaters(unsigned short player)
{

        bottle[player].contains_floaters=0;

        for ( y=BOTTLE_DEPTH-2 ; y > 0 ; y--  )
        {
                for ( x=0 ; x< BOTTLE_WIDTH ; x++ )
                {

                        switch (bottle[player].space[x][y])
                        {

//                                case VIRUS_RED1: case VIRUS_RED2: case VIRUS_BLUE1: case VIRUS_BLUE2: case VIRUS_YELLOW1: case VIRUS_YELLOW2:
//                                        break;

//                                case 0:
//                                        break;

                                case SINGLE_RED: case SINGLE_BLUE: case SINGLE_YELLOW:
                                        // nothing under me ?
                                        if (  !bottle[player].space[x][y+1] )
                                        {
                                                DropFilledSpace(player,x,y);
                                                bottle[player].contains_floaters=1;
                                        }
                                        break;

                                case LEFT_BLUE:
                                case LEFT_RED:
                                case LEFT_YELLOW:
                                        // nothing under me or down right  ?
                                        if( (  !bottle[player].space[x][y+1]  ) &&   (   !bottle[player].space[x+1][y+1]  )  )
                                        {
                                                        DropFilledSpace(player,x,y);
                                                        DropFilledSpace(player,x+1,y);
                                                        bottle[player].contains_floaters=1;
                                        }
                                        break;

                                case BOTTOM_BLUE:
                                case BOTTOM_RED:
                                case BOTTOM_YELLOW:

                                        // nothing under me ?
                                        if ( !bottle[player].space[x][y+1] )
                                        {

                                                DropFilledSpace(player,x,y);
                                                DropFilledSpace(player,x,y-1);
                                                bottle[player].contains_floaters=1;
                                        }
//                                        break;
                        }

                }
        }



        if ( ! bottle[player].contains_floaters )
        {
                CheckMatching4(player);

                // send trash if needed
                if ( bottle[player].sequences_count > 1 )
                {
                        QueTrash(player, bottle[player].sequences_count);
                }

                bottle[player].sequences_count = 0;
        }

//                bottle[player].sequences_count = 0;
}






void InitBottle(unsigned short player)
{
        unsigned short virus_type;

        unsigned short column = player ? BOTTLE2_FIRST_COLUMN : BOTTLE_FIRST_COLUMN;

        for ( x=0 ; x< BOTTLE_WIDTH ; x++ )
        {
                for ( y=0 ; y< BOTTLE_DEPTH ; y++ )
                {
                        bottle[player].space[x][y]=0;
                }
        }

        bottle[player].check_for_sequences=0;
        bottle[player].virus_count=0;
        bottle[player].contains_floaters=0;
        bottle[player].sequences_count=0;
        bottle[player].dissapear_anim=0;

        // desired amount of viruses
        z = 4 * difficulty_level[player] ;

        while ( bottle[player].virus_count < z )
        {

                x= rand() % BOTTLE_WIDTH;
                // if difficulty is greater than 15, give the viruses more possible space
                y= (difficulty_level[player] > 15) ? ( 6 + ( rand() % ( BOTTLE_DEPTH - 6) ) ) : ( 7 + ( rand() % ( BOTTLE_DEPTH - 7) ) );

                virus_type = VIRUS_RED1+(rand() % 5);

                if ( !  bottle[player].space[x][y])
                {
                        if (CheckColorMatch (virus_type, bottle[player].space[x-1][y]) && CheckColorMatch (virus_type, bottle[player].space[x-2][y]))
                        {}
                        else if (CheckColorMatch (virus_type, bottle[player].space[x][y-1]) && CheckColorMatch (virus_type, bottle[player].space[x][y-2]))
                        {}
                        else if (CheckColorMatch (virus_type, bottle[player].space[x+1][y]) && CheckColorMatch (virus_type, bottle[player].space[x+2][y]))
                        {}
                        else if (CheckColorMatch (virus_type, bottle[player].space[x][y+1]) && CheckColorMatch (virus_type, bottle[player].space[x][y+2]))
                        {}
                        else if (CheckColorMatch (virus_type, bottle[player].space[x+1][y]) && CheckColorMatch (virus_type, bottle[player].space[x-1][y]))
                        {}
                        else if (CheckColorMatch (virus_type, bottle[player].space[x][y+1]) && CheckColorMatch (virus_type, bottle[player].space[x][y-1]))
                        {}
                        else
                        {
                                bottle[player].virus_count++;
                                SetTile(column+x, BOTTLE_FIRST_ROW+y, bottle[player].space[x][y]= virus_type);
                        }
                }
        }



        PrintByte(bottle[player].virus_count, (player ? VIRUS_LEFT_COLUMN2 : VIRUS_LEFT_COLUMN1), VIRUS_LEFT_ROW);

}




void AnimateVirus()
{

int c;
        if ( virus_anim_counter < 15 )
        {
                 virus_anim_counter++;
        }
        else
        {
                virus_anim_counter=0;

                for ( x=0 ; x< BOTTLE_WIDTH ; x++ )
                {
                        for ( y=0 ; y< BOTTLE_DEPTH ; y++ )
                        {
/*                                switch ( bottle[PLAYER1].space[x][y] )
                                {
                                        case VIRUS_RED1:
                                        case VIRUS_BLUE1:
                                        case VIRUS_YELLOW1:

                                        case VIRUS_RED2:
                                        case VIRUS_BLUE2:
                                        case VIRUS_YELLOW2:*/
if (((c = bottle[PLAYER1].space[x][y]) >= VIRUS_RED1) && (c <= VIRUS_YELLOW2)) {

                                                SetTile(BOTTLE_FIRST_COLUMN + x,BOTTLE_FIRST_ROW + y, (bottle[PLAYER1].space[x][y] ^= 1));
//                                                SetTile(BOTTLE_FIRST_COLUMN + x,BOTTLE_FIRST_ROW + y,c = (bottle[PLAYER1].space[x][y] ^= 1));
//                                                SetTile(BOTTLE_FIRST_COLUMN + x,BOTTLE_FIRST_ROW + y,c);
//                                                break;


//                                                SetTile(BOTTLE_FIRST_COLUMN + x,BOTTLE_FIRST_ROW + y,c = --bottle[PLAYER1].space[x][y]);
//                                                SetTile(BOTTLE_FIRST_COLUMN + x,BOTTLE_FIRST_ROW + y,c);
//                                                break;

                                }

/*                                switch ( bottle[PLAYER2].space[x][y] )
                                {
                                        case VIRUS_RED1:
                                        case VIRUS_BLUE1:
                                        case VIRUS_YELLOW1:

                                        case VIRUS_RED2:
                                        case VIRUS_BLUE2:
                                        case VIRUS_YELLOW2:*/
if (((c = bottle[PLAYER2].space[x][y]) >= VIRUS_RED1) && (c <= VIRUS_YELLOW2)) {

                                                SetTile(BOTTLE2_FIRST_COLUMN + x,BOTTLE_FIRST_ROW + y,(bottle[PLAYER2].space[x][y] ^= 1));
//                                                SetTile(BOTTLE2_FIRST_COLUMN + x,BOTTLE_FIRST_ROW + y,c = (bottle[PLAYER2].space[x][y] ^= 1));
//                                                SetTile(BOTTLE2_FIRST_COLUMN + x,BOTTLE_FIRST_ROW + y,c);
//                                                break;


//                                                SetTile(BOTTLE2_FIRST_COLUMN + x,BOTTLE_FIRST_ROW + y,c = --bottle[PLAYER2].space[x][y]);
//                                                SetTile(BOTTLE2_FIRST_COLUMN + x,BOTTLE_FIRST_ROW + y,c);
//                                                break;

                                }

                        }
                }


        }

}

void RoundOver()
{

                        if ( round_lost[PLAYER1] )
                        {

                                Print(strLoser,BOTTLE_FIRST_COLUMN+1, BOTTLE_FIRST_COLUMN+2);
                                Print(strWinner,BOTTLE_FIRST_COLUMN+25, BOTTLE_FIRST_COLUMN+2);

                                // score board
                                Print(strWin,21,20-(++rounds_won[PLAYER2]*3));

                                if ( rounds_won[PLAYER2] == 3 ) Print("P2 WINS!",16,8);

                        }
                        else
                        {
                                Print(strWinner,BOTTLE_FIRST_COLUMN+1, BOTTLE_FIRST_COLUMN+2);
                                Print(strLoser,BOTTLE_FIRST_COLUMN+25, BOTTLE_FIRST_COLUMN+2);

                                // score board
                                Print(strWin,16,20-(++rounds_won[PLAYER1]*3));

                                if ( rounds_won[PLAYER1] == 3 ) Print("P1 WINS!",16,8);
                        }


                        // make "press start" blink untill someone presses start
                        do
                        {
/*                                if (!(long_timer&0x1FFF))
                                {

                                        Print(strPress,BOTTLE_FIRST_COLUMN+2, BOTTLE_FIRST_COLUMN+8);
                                        Print(strStart,BOTTLE_FIRST_COLUMN+2, BOTTLE_FIRST_COLUMN+9);

                                        Print(strPress,BOTTLE_FIRST_COLUMN+26, BOTTLE_FIRST_COLUMN+8);
                                        Print(strStart,BOTTLE_FIRST_COLUMN+26, BOTTLE_FIRST_COLUMN+9);

                                }
*/

                                if (long_timer == 10000)
                                {

                                        Print(strPress,BOTTLE_FIRST_COLUMN+2, BOTTLE_FIRST_COLUMN+8);
                                        Print(strStart,BOTTLE_FIRST_COLUMN+2, BOTTLE_FIRST_COLUMN+9);

                                        Print(strPress,BOTTLE_FIRST_COLUMN+26, BOTTLE_FIRST_COLUMN+8);
                                        Print(strStart,BOTTLE_FIRST_COLUMN+26, BOTTLE_FIRST_COLUMN+9);

                                }
                                if (long_timer == 20000)
                                {

                                        Print(strBlank,BOTTLE_FIRST_COLUMN+2, BOTTLE_FIRST_COLUMN+8);
                                        Print(strBlank,BOTTLE_FIRST_COLUMN+2, BOTTLE_FIRST_COLUMN+9);

                                        Print(strBlank,BOTTLE_FIRST_COLUMN+26, BOTTLE_FIRST_COLUMN+8);
                                        Print(strBlank,BOTTLE_FIRST_COLUMN+26, BOTTLE_FIRST_COLUMN+9);

                                        long_timer = 0;
                                }

                                long_timer++;

                        } while ( ! ( ReadJoypad(PLAYER1) & BTN_START ) );

                        while (ReadJoypad(PLAYER1));

}


void main(){

        unsigned short i=0;
        unsigned short j=0;


        while ( 1 )
        {

                SetTileTable(drmario_title);
                DrawMap(drmario_map_title);

Print(strPress,15,18); Print(strStart,21,18);

                while ( grab_input(PLAYER1) != MOVE_START );

                while(ReadJoypad(PLAYER1)); //wait for key release


                SetStageSettings();


                rounds_won[PLAYER1] = 0;
                rounds_won[PLAYER2] = 0;

                while ( ( rounds_won[PLAYER1] < 3) && ( rounds_won[PLAYER2] < 3))
                {

                        round_lost[PLAYER1] = 0;
                        round_lost[PLAYER2] = 0;

                        SetTileTable(main_board);
                        DrawMap(drmario_map_main);

                        // redraw score board
                        for (i=0 ; i < rounds_won[PLAYER1] ; i++ )
                                Print(strWin,16,17-(i*3));
                        for (i=0 ; i < rounds_won[PLAYER2] ; i++ )
                                Print(strWin,21,17-(i*3));

                        SetTileTable(pills);

                        InitBottle(PLAYER1);
                        InitBottle(PLAYER2);

                        NextPill(PLAYER1);
                        NewPill(PLAYER1);

                        NextPill(PLAYER2);
                        NewPill(PLAYER2);

                        round_time = 0;
                        i=0;
                        while ( ! ( round_lost[PLAYER1] || round_lost[PLAYER2] ) )
                        {

                                if(GetVsyncFlag()) // thx uze !
                                {

                                        ClearVsyncFlag();

/*                                        // you win if all viruses are gone
                                        if ( bottle[PLAYER1].virus_count == 0 )
                                        {
                                                // player 2 loses
                                                round_lost[PLAYER2]=1;
                                        }
                                        if ( bottle[PLAYER2].virus_count == 0 )
                                        {
                                                // player 1 loses
                                                round_lost[PLAYER1]=1;
                                        }*/

                                        // you win if all viruses are gone
                                        round_lost[PLAYER2] = (bottle[PLAYER1].virus_count == 0);
                                        round_lost[PLAYER1] = (bottle[PLAYER2].virus_count == 0);

                                        // check for squences if needed
                                        while  ( bottle[PLAYER1].check_for_sequences ) CheckMatching4(PLAYER1);

                                        while  ( bottle[PLAYER2].check_for_sequences ) CheckMatching4(PLAYER2);


                                        // always anymate virus
                                        AnimateVirus();


                                        // drop floaters unless pills dissapear ainmation is going on
                                        if ( drop_floater_timer < 10 )
                                        {
                                                drop_floater_timer++;
                                        }
                                        else
                                        {
                                                drop_floater_timer=0;

                                                if ( bottle[PLAYER1].contains_floaters )
                                                {
                                                         if ( ! bottle[PLAYER1].dissapear_anim )
                                                                DropFloaters(PLAYER1);
                                                }

                                                if ( bottle[PLAYER2].contains_floaters )
                                                {
                                                        if ( ! bottle[PLAYER2].dissapear_anim )
                                                                DropFloaters(PLAYER2);
                                                }
                                        }


                                        // until gravity timer hits
                                        if ( i > gravity - 6 * (round_time/1800))
                                        {
                                                // force move down when time hits
                                                MovePill(PLAYER1, MOVE_DOWN);
                                                i=0;
                                        }
                                         // move pill around unless some pills are dissapearing or there are floaters
                                        else if ( bottle[PLAYER1].dissapear_anim)
                                        {
                                                DissapearAnim(PLAYER1);
                                        }
                                        else if (bottle[PLAYER1].contains_floaters)
                                        {
                                        }
                                        else if (bottle[PLAYER1].new_pill)
                                        {
                                                if (bottle[PLAYER1].qued_trash > 0)
                                                {
                                                        FlingTrash(PLAYER1);
                                                }
                                                else
                                                {
                                                        NewPill(PLAYER1);
                                                        bottle[PLAYER1].new_pill = false;
                                                        i=0;
                                                }
                                        }
                                        else
                                        {
                                           if ( (move_type=grab_input(PLAYER1)) ) MovePill(PLAYER1, move_type);
                                           i++;
                                        }


                                        if ( j > gravity - 6 * (round_time/1800))
                                        {
                                                // force move down when time hits
                                                MovePill(PLAYER2, MOVE_DOWN);
                                                j=0;
                                        }
                                        else if ( bottle[PLAYER2].dissapear_anim)
                                        {
                                                        DissapearAnim(PLAYER2);
                                        }
                                        else if ( bottle[PLAYER2].contains_floaters )
                                        {
                                        }
                                        else if (bottle[PLAYER2].new_pill)
                                        {
                                                if (bottle[PLAYER2].qued_trash > 0)
                                                {
                                                        FlingTrash(PLAYER2);
                                                }
                                                else
                                                {
                                                        NewPill(PLAYER2);
                                                        bottle[PLAYER2].new_pill = false;
                                                        j = 0;
                                                }
                                        }
                                        else
                                        {
                                           if ( (move_type=grab_input(PLAYER2)) ) MovePill(PLAYER2, move_type);
                                           j++;
                                        }
                                        round_time++;


                                }

                        }

                        RoundOver();

                }

        } // end while (1)


}



